import 'dart:async';
import 'dart:convert';
import 'dart:math' as math;

import 'package:dio/dio.dart';

class BetterLogInterceptor extends Interceptor {
  BetterLogInterceptor({
    this.request = true,
    this.requestHeader = true,
    this.requestBody = false,
    this.responseHeader = true,
    this.responseBody = false,
    this.error = true,
    this.logSize = 1024,
  });

  /// Print request [Options]
  bool request;

  /// Print request header [Options.headers]
  bool requestHeader;

  /// Print request data [Options.data]
  bool requestBody;

  /// Print [Response.data]
  bool responseBody;

  /// Print [Response.headers]
  bool responseHeader;

  /// Print error message
  bool error;

  /// Log size per print
  final logSize;

  @override
  Future<dynamic> onRequest(RequestOptions options) async{
    String reqLog =
        ">>>>> request  ${options.method}  ${options?.uri}";

    if (request) {
      reqLog += "\n" + 'contentType：' + options.contentType?.toString();
      reqLog += "\n" + 'responseType：' + options.responseType?.toString();
      reqLog += "\n" + 'followRedirects：' + options.followRedirects?.toString();
      reqLog += "\n" + 'connectTimeout：' + options.connectTimeout?.toString();
      reqLog += "\n" + 'receiveTimeout：' + options.receiveTimeout?.toString();
      reqLog += "\n" + 'extra：' + options.extra?.toString();
    }
    if (requestHeader) {
      options.headers?.forEach((k, v) {
        reqLog += "\n$k: $v";
      });
    }
    if (requestBody) {
      reqLog += "\n\n" + options.data.toString();
    }
    print(reqLog + "\n");
    print("");
  }

  @override
  Future<dynamic> onError(DioError err) async {
    if (error) {
      print('*** DioError ***:');
      print(err);
      if (err.response != null) {
        _printResponse(err.response);
      }
    }
  }

  @override
  Future<dynamic> onResponse(Response response) async {
    _printResponse(response);
  }

  void _printResponse(Response response) {
    String responseLog =
        "<<<<< response  ${response.statusCode}  ${response?.request?.uri}";
    if (responseHeader) {
      if (response.isRedirect) printKV('redirect', response.realUri);
      responseLog +=
          "\n" + response?.headers?.toString()?.replaceAll("\n", "\n");
    }
    responseLog += "\n" ;
    if (responseBody) {
      if (response?.headers['content-type'].toString().contains('application/json')||
          response?.headers['Content-Type'].toString().contains('application/json')) {
        responseLog += JsonEncoder.withIndent("  ")
                .convert(jsonDecode(response.toString()));
      } else {
        responseLog += response.toString();
      }
    }

    if (responseLog.length > logSize) {
      int n = responseLog.length ~/ logSize;
      if (responseLog.length % logSize > 0) n += 1;
      int start = 0;
      for (int i = 0; i < n; i++){
        print(responseLog.substring(start, (i==(n-1)) ? responseLog.length : (start+logSize)));
        start += logSize;
      }
    } else {
      print(responseLog + "\n");
    }
    print("");
  }

  printKV(String key, Object v) {
    print('$key: $v');
  }

  printAll(msg) {
    msg.toString().split("\n").forEach(_printAll);
  }

  _printAll(String msg) {
    int groups = (msg.length / logSize).ceil();
    for (int i = 0; i < groups; ++i) {
      print((i > 0 ? '<<Log follows the previous line: ' : '') +
          msg.substring(
              i * logSize, math.min<int>(i * logSize + logSize, msg.length)));
    }
  }
}
